@@ -3,6 +3,9 @@
 Created on April 13,  2022
 
 @author: wang Haihua
+Refer to 
+1. https://github.com/LinBaiTao/Supplier_Selection_Methods 
+2.https://github.com/akestoridis/mcdm
 """
 
 from importlib_metadata import entry_points
@@ -10,7 +13,7 @@ import pandas as pd
 import numpy as np
 from collections import Counter
 import matplotlib.pyplot as plt
-plt.rcParams.update({'font.family': 'STIXGeneral', 'mathtext.fontset': 'stix'})
+plt.rcParams.update({'font.family': 'SimHei', 'mathtext.fontset': 'stix'})
 
 ###########################################################################################
 ###############################     1 Preprocessing         ###############################
@@ -173,7 +176,7 @@ def Convert_Higher_is_Better(df, columns, conversion_type = 'absolute'):
     conversion_type : a string specifying the type of conversion to perform
         - 'absolute' converts based on absolute scale
         - 'relative' converts based on relative scale
-        - 'inverse' converts  the raw value to its inverse
+        - 'inverse'  converts based on its inverse value
         
     Yields
     ------
@@ -184,19 +187,19 @@ def Convert_Higher_is_Better(df, columns, conversion_type = 'absolute'):
     temp_df = df.copy()
         
     for column in columns:
-        if conversion_type is 'absolute':
+        if conversion_type.lower() is 'absolute':
             new_column = column+' (absolute HIB)'
             max_entry = temp_df[column].max()
             temp_df[new_column] = max_entry - temp_df[column]
     
-        elif conversion_type is 'relative':
+        elif conversion_type.lower() is 'relative':
             new_column = column+' (relative HIB)'
             min_entry = temp_df[column].min()
             max_entry = temp_df[column].max()
             temp_df[new_column] = (max_entry - temp_df[column])/(max_entry-min_entry)
-
-        elif conversion_type is 'inverse':
-            new_column = column + '(invers HIB)'
+        
+        elif conversion_type.lower() is 'inverse':
+            new_column = column+' (inverse HIB)'
             temp_df[new_column] = 1/temp_df[column]
 
         else:
@@ -212,28 +215,57 @@ def Convert_Higher_is_Better(df, columns, conversion_type = 'absolute'):
 ####***************************     2.1 Correlation         ****************************###
 
 def abspearson(z_matrix):
+    """ Calculate the absolute value of the Pearson correlation of the previded matrix.
+
+    Parameters
+    ----------
+    z_matrix : a pandas or array object that contains specific columns
+
+    
+    Yields:
+    -------
+    abs_corr : a matrix of the Pearson correlation coefficients
+
     """
-    Return the absolute value of the Pearson correlation coefficients of the
-    provided matrix.
-    """
+    
     # Make sure that the provided matrix is a float64 NumPy array
     z_matrix = np.array(z_matrix, dtype=np.float64)
 
-    return np.absolute(np.corrcoef(z_matrix, rowvar=False))
+    abs_corr = np.absolute(np.corrcoef(z_matrix, rowvar=False))
+
+    return abs_corr
 
 
 def pearson(z_matrix):
-    """
-    Return the Pearson correlation coefficients of the provided matrix.
+    """ Calculate the Pearson correlation coefficient of the provided matrix
+
+    Parameters :
+    ----------
+    z_matrix : a pandas or array object that contains specific columns
+
+    Yields:
+    -------
+    p_corr : the Pearson correlation coefficients of the provided matrix.
     """
     # Make sure that the provided matrix is a float64 NumPy array
     z_matrix = np.array(z_matrix, dtype=np.float64)
 
-    return np.corrcoef(z_matrix, rowvar=False)
+    p_corr = np.corrcoef(z_matrix, rowvar=False)
+    return p_corr
+
 
 def dcor(z_matrix):
-    """
-    Return the distance correlation coefficients of the provided matrix.
+    """ Calculate the distance correlation coefficients of the provided matrix
+    
+    Parameters :
+    ----------
+
+    z_matrix : a pandas or array object that contains specific columns
+
+    Yields:
+    -------
+    dcor_matrix : the distance correlation matrix
+
     """
     # Make sure that the provided matrix is a float64 NumPy array
     z_matrix = np.array(z_matrix, dtype=np.float64)
@@ -276,9 +308,18 @@ def dcor(z_matrix):
 
 
 def squared_dcov_matrix(z_matrix):
-    """
-    Return the matrix of squared distance covariance between the columns of
+    """ Calculate the matrix of squared distance covariance between the columns of
     the provided matrix.
+    
+    Parameters :
+    ----------
+
+    z_matrix : a pandas or array object that contains specific columns
+
+    Yields:
+    -------
+    dcor2_matrix : the matrix of squared distance covariance
+
     """
     # Initialize the distance covariance matrix
     dcov2_matrix = np.zeros(
@@ -312,8 +353,17 @@ def squared_dcov_matrix(z_matrix):
 
 
 def dist_matrix(z_vector):
-    """
-    Return the Euclidean distance matrix of the provided vector.
+    """ Calculate the Euclidean distance matrix of provided vector
+    
+    Parameters :
+    ----------
+
+    z_vector : a pandas or array object that contains specific columns
+
+    Yields:
+    -------
+    d_matrix : the matrix of the Euclidean distance matrix
+
     """
     # Initialize the Euclidean distance matrix
     dmatrix = np.zeros(
@@ -332,35 +382,79 @@ def dist_matrix(z_vector):
 
 
 def lin_func(dmatrix):
+    """ Calculate the result of the linear function for the provided distance matrix
+    
+    Parameters :
+    ----------
+
+    d_matrix : a pandas or array object that contains specific columns
+
+    Yields:
+    -------
+    res : the result of the linear function for the provided distance matrix
+.
     """
-    Return the result of the linear function for the provided distance matrix.
-    """
-    return (
-        dmatrix
-        - np.mean(dmatrix, axis=0)
-        - np.reshape(np.mean(dmatrix, axis=1), (dmatrix.shape[0], 1))
-        + np.mean(dmatrix)
-    )
+    res = dmatrix - np.mean(dmatrix, axis=0) - np.reshape(np.mean(dmatrix, axis=1), (dmatrix.shape[0], 1))+ np.mean(dmatrix)
+
+    return res
 
 
 def squared_dcov(j_func, l_func):
+    """ Calculate the squared distance covariance between the corresponding columns
+    
+    Parameters :
+    ----------
+    j_func : an array type object
+    l_func : another array type object
+
+    Yields:
+    -------
+    sq_dcov : the squared distance covariance of j_func and l_func
+
     """
-    Return the squared distance covariance between the corresponding columns.
-    """
-    return np.sum(np.multiply(j_func, l_func)) / (j_func.shape[0] ** 2)
+    sq_dov = np.sum(np.multiply(j_func, l_func)) / (j_func.shape[0] ** 2)
+    return sq_dov
 
 
 def squared_dcor(jl_dcov2, j_dvar2, l_dvar2):
+
+    """ Calculate the squared distance correlation between the corresponding columns
+    
+    Parameters :
+    ----------
+    j1_dov2 : an array
+
+    j_dvar2 : an array
+
+    l_dvar2 : an array
+
+    Yields:
+    -------
+    sq_dcor : the squared distance correlation
+
     """
-    Return the squared distance correlation between the corresponding columns.
-    """
-    return jl_dcov2 / np.sqrt(j_dvar2 * l_dvar2)
+    sq_dcor = jl_dcov2 / np.sqrt(j_dvar2 * l_dvar2)
+    return sq_dcor
 
 
 
 def correlate(z_matrix, c_method):
-    """
-    Return the selected correlation coefficients of the provided matrix.
+    """ Calculate the selected correlation coefficients of the provide matrix
+    
+    Parameters :
+    ----------
+
+    z_matrix : a pandas or array object that contains specific columns
+
+    c_method : the selected correlation method
+        - "PEARSON" - Pearson correlation
+        - "ABSPEARSON" - Absolute Pearson correlation
+        - "DCOR" - Distance correlation
+
+    Yields:
+    -------
+    the selected correlation cofficients
+
     """
     # Use the selected correlation method
     if c_method.upper() == "PEARSON":
@@ -376,9 +470,23 @@ def correlate(z_matrix, c_method):
 ####***********************    2.2 Critic                     ****************###
 
 def critic(z_matrix, c_method="Pearson"):
-    """
-    Return the weight vector of the provided decision matrix using the
-    Criteria Importance Through Intercriteria Correlation method.
+    """ Calculate the  weight vector of the provided matrix using the Criteria 
+    Importance Through Intercriteria Correlation(CRITIC) method
+    
+    Parameters :
+    ----------
+
+    z_matrix : a pandas or array object that contains specific columns
+
+    c_method : the selected correlation method
+        - "PEARSON" - Pearson correlation
+        - "ABSPEARSON" - Absolute Pearson correlation
+        - "DCOR" - Distance correlation
+
+    Yields:
+    -------
+    critic_weights : the weight vector of CRITIC
+
     """
     # Perform sanity checks
     z_matrix = np.array(z_matrix, dtype=np.float64)
@@ -396,17 +504,27 @@ def critic(z_matrix, c_method="Pearson"):
         for l_col in range(z_matrix.shape[1]):
             tmp_sum += 1.0 - corr_matrix[j_col, l_col]
         imp_vector[j_col] = sd_vector[j_col] * tmp_sum
-
+    
     # Normalize the importance of each criterion
-    return imp_vector / np.sum(imp_vector)
+    critic_weights = imp_vector / np.sum(imp_vector)
+    return critic_weights
 
 
-####***********************           2.2 Entropy                        ****************###
+####***********************           2.3 Entropy                        ****************###
 
 def em(z_matrix):
-    """
-    Return the weight vector of the provided decision matrix using the Entropy
-    Measure method.
+    """ Calculate the weight vector of the provided decision matrix using the Entropy
+    Measure(EM) method
+    
+    Parameters :
+    ----------
+
+    z_matrix : a pandas or array object that contains specific columns
+
+    Yields:
+    -------
+    em_weights : the weight vector of EM
+
     """
     # Perform sanity checks
     z_matrix = np.array(z_matrix, dtype=np.float64)
@@ -425,32 +543,50 @@ def em(z_matrix):
 
     # The importance of each criterion corresponds to
     # its normalized degree of divergence
-    return (1.0 - e_vector) / np.sum(1.0 - e_vector)
+    em_weights = (1.0 - e_vector) / np.sum(1.0 - e_vector)
+    return em_weights
 
 
-####***********************           2.3 Mean Weighting               ****************###
+####***********************           2.4 Mean Weighting               ****************###
 def mw(z_matrix):
-    """
-    Return the weight vector of the provided decision matrix using the Mean
-    Weights method.
+    """ Calculate the the weight vector of the provided matrix using the Mean
+    Weights(MW) method
+    
+    Parameters :
+    ----------
+
+    z_matrix : a pandas or array object that contains specific columns
+
+    Yields:
+    -------
+    mw_weights : the weight vector of MW
+
     """
     # Perform sanity checks
     z_matrix = np.array(z_matrix, dtype=np.float64)
 
     # Each criterion is considered equally important
-    return (
-        np.full(z_matrix.shape[1], 1.0 / z_matrix.shape[1], dtype=np.float64)
-    )
+    mw_weights = np.full(z_matrix.shape[1], 1.0 / z_matrix.shape[1], dtype=np.float64)
+    return mw_weights
 
 
-####***********************           2.3 Standard Deviation Weighting               ****************###
+####***********************           2.5 Standard Deviation Weighting               ****************###
 
 def sd(z_matrix):
+    """ Calculate the the weight vector of the provided matrix using the 
+    Standard Deviation(SD) method
+    
+    Parameters :
+    ----------
+
+    z_matrix : a pandas or array object that contains specific columns
+
+    Yields:
+    -------
+    sd_weights : the weight vector of SD
+
     """
-    Return the weight vector of the provided decision matrix using the
-    Standard Deviation method.
-    """
-    # Perform sanity checks
+
     z_matrix = np.array(z_matrix, dtype=np.float64)
 
     # Compute the standard deviation of each criterion
@@ -458,17 +594,33 @@ def sd(z_matrix):
 
     # The importance of each criterion corresponds to
     # its normalized standard deviation
-    return sd_vector / np.sum(sd_vector)
+    sd_weights = sd_vector / np.sum(sd_vector)
+    return sd_weights
 
 
-####******************   2.4  Variability and Interdependencies of Criteria method    ****************###
+####******************   2.6  Variability and Interdependencies of Criteria method    ****************###
 
 def vic(z_matrix, c_method="dCor"):
+    """ Calculate the the weight vector of the provided matrix using the 
+    Variablity and Interdependencies of Criteria(VIC) method.
+    
+    Parameters :
+    ----------
+
+    z_matrix : a pandas or array object that contains specific columns
+
+    c_method : the selected correlation method
+        - "DCOR" - Distance correlation (default parameter)
+        - "PEARSON" - Pearson correlation
+        - "ABSPEARSON" - Absolute Pearson correlation
+        
+
+    Yields:
+    -------
+    vic_weights : the weight vector of VIC
+
     """
-    Return the weight vector of the provided decision matrix using the
-    Variability and Interdependencies of Criteria method.
-    """
-    # Perform sanity checks
+
     z_matrix = np.array(z_matrix, dtype=np.float64)
     if c_method is None:
         c_method = "dCor"
@@ -491,6 +643,34 @@ def vic(z_matrix, c_method="dCor"):
     return imp_vector / np.sum(imp_vector)
 
 
+####******************   2.7  Analytic Hierachy Processing    ****************###
+def ahp(judge_matrix):
+    """ Calculate the weights using Analytic Hierachy Processing(AHP)
+    
+    Parameters :
+    ----------
+
+    judge_matrix : a pair-wise matrix
+        
+    Yields:
+    -------
+    ahp_weights : the weight vector of AHP
+
+    """
+
+    judge_matrix = np.array(judge_matrix, dtype=np.float64)
+    n =  judge_matrix.shape[0]
+    eigenvalues, eigenvectors = np.linalg.eig(judge_matrix)
+    value_max = eigenvalues.max()
+    CI = (value_max-n)/(n-1)
+    RI_list = [0,0,0,0.58,0.9,1.12,1.24,1.32,1.41,1.45,1.49] 
+    if CI/RI_list[n]>0.1:
+        return 'Judgement Matrix is NOT consistent'
+    else:
+        print('The Max Eigenvalue is ',value_max)
+        ahp_weights = eigenvectors/eigenvectors.sum()
+        return ahp_weights
+        
 
 ###########################################################################################
 ###############################        3 MCDM Models        ###############################
